/********************************************************
 Name          : main.c
 Author        : MoroboshiLUM
 Copyright     :
 Description   : EVK1101 template
 **********************************************************/

// Include Files
#include <avr32/io.h>
#include "compiler.h"
#include "board.h"

/* TODO: Add software framework include drivers below */
#include "gpio.h"
#include "spi.h"
#include "rtc.h"
#include "pm.h"

// wait is timeout * 62us
static void wait(int timeout)
{
	volatile unsigned int start,now;

	start=rtc_get_value(&AVR32_RTC);
	while (1) {
	  now=rtc_get_value(&AVR32_RTC);
	  now=(now-start) & 0x7FFFFFFF;
	  if (now>timeout) break;
	}
}

// send command to OLED
static void send_cmd(unsigned char cmd)
{
	gpio_clr_gpio_pin(AVR32_PIN_PB04);		// cmd select
	while (spi_write(&AVR32_SPI,cmd)!=SPI_OK) ;
}

// send data to OLED
static void send_data(unsigned char data)
{
	gpio_set_gpio_pin(AVR32_PIN_PB04);		// data select
	while (spi_write(&AVR32_SPI,data)!=SPI_OK) ;
}

static void init_oled(void)
{
	wait(2);
    gpio_set_gpio_pin(AVR32_PIN_PA10);		// RESET de-Assert
    gpio_clr_gpio_pin(AVR32_PIN_PB05);		// VDD ON
	wait(1000); // 600ms wait
	gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select
	spi_selectChip(&AVR32_SPI,2); 			// select cs2

	send_cmd(0xAE); // Display Off
	send_cmd(0x15); // Column Address
	send_cmd(0x00); // Set
	send_cmd(0x5F); // Set
	send_cmd(0x75); // Row Addess
	send_cmd(0x00); // Set
	send_cmd(0x3F); // Set
	send_cmd(0x81); // Set Contrast for Color A
	send_cmd(0x5A); // Set
	send_cmd(0x82); // Set Contrast for Color B
	send_cmd(0x2C); // Set
	send_cmd(0x83); // Set Contrast for Color C Now setting value is 15V, 100cd.
	send_cmd(0x54); // Set If you want to change
	send_cmd(0x87); // Master Current Control the OLED Brightnees.
	send_cmd(0x09); // Set Pls see the Note 1)
	send_cmd(0xA0); // Set Re-map & Data Format
	send_cmd(0x70); // Set
	send_cmd(0xA1); // Set Display Start Line
	send_cmd(0x00); // Set
	send_cmd(0xA2); // Set off Set
	send_cmd(0x00); // Set
	send_cmd(0xA8); // Multiplex Ratio
	send_cmd(0x3F); // Set
	send_cmd(0xAD); // Master Configuration
	send_cmd(0x8E); // Set
	send_cmd(0xB1); // Adjust Driving period
	send_cmd(0x11); // Set
	send_cmd(0xB3); // Colock Divider Oscillator Frequency
	send_cmd(0xD0); // Set
	send_cmd(0xBB); // Pre_charge Voltage Setting
	send_cmd(0x00); // Set
	send_cmd(0xBC); // Pre_charge Voltage Setting Now setting value is 15V, 100cd.
	send_cmd(0x2A); // Set If you want to change
	send_cmd(0xBD); // Pre_charge Voltage Setting the OLED Brightnees.
	send_cmd(0x00); // Set Pls see the Note 1)
	send_cmd(0xBE); // Set VCOMH Level
	send_cmd(0x3F); // Set
	send_cmd(0xB9); // Enable Linear Gray Scale
	send_cmd(0xAF); // Display On
	send_cmd(0x15); //Set Column Address RAM
	send_cmd(0x00); //Dec->0
	send_cmd(0x5F); //Dec->95
	send_cmd(0x75); //Set Row RAM Address
	send_cmd(0x00); //Dec->0
	send_cmd(0x3F); //Dec->63
}

static void init_gpio(void)
{
  static const gpio_map_t SPI_GPIO_MAP =
  {
    {AVR32_SPI_NPCS_2_0_PIN, AVR32_SPI_NPCS_2_0_FUNCTION},
    {AVR32_SPI_MOSI_0_0_PIN, AVR32_SPI_MOSI_0_0_FUNCTION},
    {AVR32_SPI_SCK_0_0_PIN, AVR32_SPI_SCK_0_0_FUNCTION}
  };

  // Switch main clock to external oscillator 0 (crystal).
  pm_switch_to_osc0(&AVR32_PM, FOSC0, OSC0_STARTUP);

  // Assign GPIO to SPI.
  gpio_enable_module(SPI_GPIO_MAP,
                     sizeof(SPI_GPIO_MAP) / sizeof(SPI_GPIO_MAP[0]));

  gpio_set_gpio_pin(AVR32_PIN_PB05);		// VDD OFF
  gpio_clr_gpio_pin(AVR32_PIN_PA10);		// RESET Assert
  gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select

  gpio_enable_pin_glitch_filter(GPIO_PUSH_BUTTON_0);
}

static void init_spi(void)
{
  static const spi_options_t SPI_OPTIONS =
  {
    .reg          = 2,
    .baudrate     = 5000000,  // 5MHz
    .bits         = 8,
    .spck_delay   = 0,
    .trans_delay  = 0,
    .stay_act     = 1,
    .spi_mode     = 0,
    .modfdis      = 1
  };

  spi_reset(&AVR32_SPI);  // spi reset
  // master reset
  spi_initMaster(&AVR32_SPI,&SPI_OPTIONS);
  // Set SPI selection mode: variable_ps, pcs_decode, delay.
  spi_selectionMode(&AVR32_SPI, 0, 0, 0);
  // enable spi
  spi_enable(&AVR32_SPI);
  // setup chip reg
  spi_setupChipReg(&AVR32_SPI,&SPI_OPTIONS,FOSC0);
  // unselect chip
  spi_selectChip(&AVR32_SPI,2); //
}

static void init_rtc(void)
{
  rtc_init(&AVR32_RTC,RTC_OSC_32KHZ, 0); // treat as 16KHz Counter
   // Enable the RTC
  rtc_enable(&AVR32_RTC);
}

extern const unsigned char GDATA[];

int main(void) {
	int x,y;
	unsigned char *p;

	/* TODO: replace this comment with your code */
	init_gpio();
	init_rtc();
	init_spi();
	init_oled();

	/****/
	spi_unselectChip(&AVR32_SPI,2);			// unselect chip
	gpio_set_gpio_pin(AVR32_PIN_PB04);		// data select
	spi_selectChip(&AVR32_SPI,2);			// cs2 select
	/****/

	p=(unsigned char*)&GDATA[0];

	for(y=0;y<64;y++) 
	{
		for(x=0;x<96;x++)
		{
  			send_data(*p++);
  			send_data(*p++);
		}
	}

  spi_unselectChip(&AVR32_SPI,2);
  gpio_clr_gpio_pin(AVR32_PIN_PB04);		// command select
  spi_selectChip(&AVR32_SPI,2);

  while (gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) ; // wait push button0
  send_cmd(0xAE); // Display Off
  gpio_set_gpio_pin(AVR32_PIN_PB05);		// VDD OFF
  wait(1000); // 600ms wait

  return 0;
}
